@@ -1,27 +1,43 @@
-
+##  Copyright (C) 2007-2019 Martin Maechler
+##
 ## nearcor.R :
-## Copyright (2007) Jens Oehlschlägel
-## GPL licence, no warranty, use at your own risk
+##  Copyright (C) 2007  Jens Oehlschlägel
+##
+##  This program is free software; you can redistribute it and/or modify
+##  it under the terms of the GNU General Public License as published by
+##  the Free Software Foundation; either version 2 of the License, or
+##  (at your option) any later version.
+##
+##  This program is distributed in the hope that it will be useful,
+##  but WITHOUT ANY WARRANTY; without even the implied warranty of
+##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+##  GNU General Public License for more details.
+##
+##  A copy of the GNU General Public License is available at
+##  https://www.R-project.org/Licenses/
 
 nearPD <-
     ## Computes the nearest correlation matrix to an approximate
     ## correlation matrix, i.e. not positive semidefinite.
-
     function(x               # n-by-n approx covariance/correlation matrix
              , corr = FALSE, keepDiag = FALSE
+             , base.matrix = FALSE # if TRUE return "base matrix" otherwise "dpoMatrix"
              , do2eigen = TRUE  # if TRUE do a sfsmisc::posdefify() eigen step
              , doSym = FALSE  # symmetrize after tcrossprod()
              , doDykstra = TRUE # do use Dykstra's correction
              , only.values = FALSE# if TRUE simply return lambda[j].
+             , ensureSymmetry = !isSymmetric(x)# so user can set to FALSE iff she knows..
              , eig.tol   = 1e-6 # defines relative positiveness of eigenvalues compared to largest
              , conv.tol  = 1e-7 # convergence tolerance for algorithm
              , posd.tol  = 1e-8 # tolerance for enforcing positive definiteness
-             , maxit    = 100 # maximum number of iterations allowed
+             , maxit    = 100L # maximum number of iterations allowed
+             , conv.norm.type = "I"
              , trace = FALSE # set to TRUE (or 1 ..) to trace iterations
              )
+
 {
-    if(!isSymmetric(x)) { ## ideally the user should do this herself
-	message("applying nearPD() to symmpart(x)")
+    if(ensureSymmetry) { ## only if needed/wanted ...
+	## message("applying nearPD() to symmpart(x)")
 	x <- symmpart(x)
     }
     n <- ncol(x)
@@ -32,7 +48,7 @@ nearPD <-
         D_S <- x; D_S[] <- 0
     }
     X <- x
-    iter <- 0 ; converged <- FALSE; conv <- Inf
+    iter <- 0L ; converged <- FALSE; conv <- Inf
 
     while (iter < maxit && !converged) {
         Y <- X
@@ -47,9 +63,10 @@ nearPD <-
 
         ## create mask from relative positive eigenvalues
         p <- d > eig.tol*d[1]
+	if(!any(p)) stop("Matrix seems negative semi-definite")
 
         ## use p mask to only compute 'positive' part
-        Q <- Q[,p,drop = FALSE]
+        Q <- Q[,p, drop = FALSE]
         ## X <- Q %*% D[p,p,drop = FALSE] %*% t(Q)  --- more efficiently :
         X <- tcrossprod(Q * rep(d[p], each=nrow(Q)), Q)
 
@@ -60,26 +77,26 @@ nearPD <-
         ## project onto symmetric and possibly 'given diag' matrices:
         if(doSym)
             X <- (X + t(X))/2
-        if(corr) diag(X) <- 1
-        else if(keepDiag) diag(X) <- diagX0
+	if(corr)
+	    diag(X) <- 1
+	else if(keepDiag)
+	    diag(X) <- diagX0
 
-        conv <- norm(Y-X, "I") / norm(Y, "I")
-        iter <- iter + 1
+        conv <- norm(Y-X, conv.norm.type) / norm(Y, conv.norm.type)
+        iter <- iter + 1L
 	if (trace)
 	    cat(sprintf("iter %3d : #{p}=%d, ||Y-X|| / ||Y||= %11g\n",
 			iter, sum(p), conv))
-
         converged <- (conv <= conv.tol)
     }
 
-    if(!converged) {
-        warning("nearPD() did not converge in ", iter, " iterations")
-    }
+    if(!converged)
+	warning(gettextf("'nearPD()' did not converge in %d iterations",
+			 iter), domain = NA)
 
     ## force symmetry is *NEVER* needed, we have symmetric X here!
     ## X <- (X + t(X))/2
-    if(do2eigen || only.values) {
-        ## begin from posdefify(sfsmisc)
+    if(do2eigen || only.values) { ## begin from posdefify(sfsmisc)
         e <- eigen(X, symmetric = TRUE)
         d <- e$values
         Eps <- posd.tol * abs(d[1])
@@ -96,14 +113,16 @@ nearPD <-
         if(only.values) return(d)
 
         ## unneeded(?!): X <- (X + t(X))/2
-        if(corr) diag(X) <- 1
-        else if(keepDiag) diag(X) <- diagX0
-
+	if(corr)
+	    diag(X) <- 1
+	else if(keepDiag)
+	    diag(X) <- diagX0
     } ## end from posdefify(sfsmisc)
 
     structure(list(mat =
-		   new("dpoMatrix", x = as.vector(X),
-		       Dim = c(n,n), Dimnames = .M.DN(x)),
+			if(base.matrix) X
+			else new("dpoMatrix", x = as.vector(X),
+				 Dim = c(n,n), Dimnames = .M.DN(x)),
                    eigenvalues = d,
                    corr = corr, normF = norm(x-X, "F"), iterations = iter,
 		   rel.tol = conv, converged = converged),
