@@ -10,9 +10,6 @@
     x
 }
 
-
-## ~~~~ COERCIONS TO ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
 setAs("numeric", "pMatrix",
       function(from) {
           J <- new("pMatrix")
@@ -37,7 +34,7 @@ setAs("nsparseMatrix", "pMatrix",
           d <- from@Dim
           if((n <- d[1L]) != d[2L])
               stop("attempt to coerce non-square matrix to pMatrix")
-          from <- .sparse2g(from)
+          from <- .M2gen(from)
           J <- new("pMatrix")
           J@Dim <- d
           J@Dimnames <- from@Dimnames
@@ -59,16 +56,8 @@ setAs("nsparseMatrix", "pMatrix",
           stop("matrix must have exactly one nonzero element in each row and column")
       })
 
-setAs("Matrix", "pMatrix",
-      function(from) as(as(from, "nsparseMatrix"), "pMatrix"))
-
-setAs("matrix", "pMatrix",
-      function(from) as(as(from, "nsparseMatrix"), "pMatrix"))
-
-
-## ~~~~ METHODS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
-## NB: pMatrix are orthogonal, hence the transpose and inverse coincide
+setAs("indMatrix", "pMatrix",
+      function(from) new("pMatrix", from))
 
 setMethod("t", signature(x = "pMatrix"),
           function(x) {
@@ -133,7 +122,7 @@ setMethod("%*%", signature(x = "pMatrix", y = "matrix"),
           function(x, y) {
               mmultDim(x@Dim, dim(y), type = 1L)
               perm <- if(x@margin == 1L) x@perm else invertPerm(x@perm)
-              r <- .m2ge(y[perm, , drop = FALSE], "d")
+              r <- .m2dense(y[perm, , drop = FALSE], "dge")
               r@Dimnames <- mmultDimnames(x@Dimnames, dimnames(y), type = 1L)
               r
           })
@@ -142,7 +131,7 @@ setMethod("%*%", signature(x = "matrix", y = "pMatrix"),
           function(x, y) {
               mmultDim(dim(x), y@Dim, type = 1L)
               perm <- if(y@margin == 1L) invertPerm(y@perm) else y@perm
-              r <- .m2ge(x[, perm, drop = FALSE], "d")
+              r <- .m2dense(x[, perm, drop = FALSE], "dge")
               r@Dimnames <- mmultDimnames(dimnames(x), y@Dimnames, type = 1L)
               r
           })
@@ -169,7 +158,7 @@ setMethod("%&%", signature(x = "pMatrix", y = "matrix"),
           function(x, y) {
               mmultDim(x@Dim, dim(y), type = 1L)
               perm <- if(x@margin == 1L) x@perm else invertPerm(x@perm)
-              r <- .m2ge(y[perm, , drop = FALSE], "n")
+              r <- .m2dense(y[perm, , drop = FALSE], "nge")
               r@Dimnames <- mmultDimnames(x@Dimnames, dimnames(y), type = 1L)
               r
           })
@@ -178,7 +167,7 @@ setMethod("%&%", signature(x = "matrix", y = "pMatrix"),
           function(x, y) {
               mmultDim(dim(x), y@Dim, type = 1L)
               perm <- if(y@margin == 1L) invertPerm(y@perm) else y@perm
-              r <- .m2ge(x[, perm, drop = FALSE], "n")
+              r <- .m2dense(x[, perm, drop = FALSE], "nge")
               r@Dimnames <- mmultDimnames(dimnames(x), y@Dimnames, type = 1L)
               r
           })
@@ -214,8 +203,8 @@ setMethod("crossprod", signature(x = "matrix", y = "pMatrix"),
           function(x, y = NULL, boolArith = NA, ...) {
               mmultDim(dim(x), y@Dim, type = 2L)
               perm <- if(y@margin == 1L) invertPerm(y@perm) else y@perm
-              r <- .m2ge(t(x)[, perm, drop = FALSE],
-                         if(isTRUE(boolArith)) "n" else "d")
+              r <- .m2dense(t(x)[, perm, drop = FALSE],
+                            if(isTRUE(boolArith)) "nge" else "dge")
               r@Dimnames <- mmultDimnames(dimnames(x), y@Dimnames, type = 2L)
               r
           })
@@ -243,8 +232,8 @@ setMethod("tcrossprod", signature(x = "pMatrix", y = "matrix"),
           function(x, y = NULL, boolArith = NA, ...) {
               mmultDim(x@Dim, dim(y), type = 3L)
               perm <- if(x@margin == 1L) x@perm else invertPerm(x@perm)
-              r <- .m2ge(t(y)[perm, , drop = FALSE],
-                         if(isTRUE(boolArith)) "n" else "d")
+              r <- .m2dense(t(y)[perm, , drop = FALSE],
+                            if(isTRUE(boolArith)) "nge" else "dge")
               r@Dimnames <- mmultDimnames(x@Dimnames, dimnames(y), type = 3L)
               r
           })
