@@ -225,7 +225,7 @@ void _PREFIX_ ## dense_unpacked_copy_diagonal(_CTYPE_ *dest, \
 		for (j = 0; j < n; ++j, dest += n1a, src += n1a) \
 			*dest = *src; \
 	} else { \
-		error(_("incompatible 'n' and 'len' to '*_copy_diagonal()'")); \
+		error(_("incompatible '%s' and '%s' in %s()"), "n", "len", __func__); \
 	} \
 	return; \
 }
@@ -375,7 +375,7 @@ SEXP unpacked_force(SEXP x, int n, char uplo, char diag)
 {
 	SEXPTYPE tx = TYPEOF(x);
 	if (tx < LGLSXP || tx > CPLXSXP)
-		ERROR_INVALID_TYPE("'x'", tx, "unpacked_force");
+		ERROR_INVALID_TYPE(x, __func__);
 	R_xlen_t nx = XLENGTH(x);
 	SEXP y = PROTECT(allocVector(tx, nx));
 
@@ -447,109 +447,6 @@ SEXP unpacked_force(SEXP x, int n, char uplo, char diag)
 	return y;
 }
 
-/* pack(x), returning packedMatrix */
-SEXP unpackedMatrix_pack(SEXP from, SEXP strict, SEXP tr_if_ge, SEXP up_if_ge)
-{
-	static const char *valid_from[] = {
-	/* 0 */ "dtrMatrix", "ltrMatrix", "ntrMatrix",
-	/* 3 */ "corMatrix", "dpoMatrix", /* must match before dsyMatrix */
-	/* 5 */ "dsyMatrix", "lsyMatrix", "nsyMatrix",
-	/* 8 */ "dgeMatrix", "lgeMatrix", "ngeMatrix", ""};
-	static const char *valid_to[] = {
-	/* 0 */ "dtpMatrix", "ltpMatrix", "ntpMatrix",
-	/* 3 */ "pcorMatrix", "dppMatrix",
-	/* 5 */ "dspMatrix", "lspMatrix", "nspMatrix", ""};
-	int ivalid = R_check_class_etc(from, valid_from);
-	if (ivalid < 0)
-		ERROR_INVALID_CLASS(from, "unpackedMatrix_pack");
-	if (asLogical(strict) == 0 && (ivalid == 3 || ivalid == 4))
-		ivalid = 5; /* corMatrix,dpoMatrix->dspMatrix */
-
-	int shift = (ivalid < 8) ? 0 : ((asLogical(tr_if_ge) != 0) ? 3+2+3 : 3);
-	SEXP to = PROTECT(NEW_OBJECT_OF_CLASS(valid_to[ivalid - shift]));
-
-	SEXP dim = PROTECT(GET_SLOT(from, Matrix_DimSym));
-	int *pdim = INTEGER(dim), n = pdim[0];
-	if (pdim[1] != n)
-		error(_("attempt to pack non-square matrix"));
-	if (n > 0)
-		SET_SLOT(to, Matrix_DimSym, dim);
-	UNPROTECT(1); /* dim */
-
-	SEXP dimnames = PROTECT(GET_SLOT(from, Matrix_DimNamesSym));
-	SET_SLOT(to, Matrix_DimNamesSym, dimnames);
-	UNPROTECT(1); /* dimnames */
-
-	SEXP uplo;
-
-	if (ivalid >= 8)
-		PROTECT(uplo = mkString((asLogical(up_if_ge) != 0) ? "U" : "L"));
-	else {
-		/* .(tr|sy)Matrix */
-		PROTECT(uplo = GET_SLOT(from, Matrix_uploSym));
-		if (ivalid < 3) {
-			/* .trMatrix */
-			SEXP diag = PROTECT(GET_SLOT(from, Matrix_diagSym));
-			char di = *CHAR(STRING_ELT(diag, 0));
-			if (di != 'N')
-				SET_SLOT(to, Matrix_diagSym, diag);
-			UNPROTECT(1); /* diag */
-		} else {
-			/* .syMatrix */
-			SEXP factors = PROTECT(GET_SLOT(from, Matrix_factorSym));
-			if (LENGTH(factors) > 0)
-				SET_SLOT(to, Matrix_factorSym, factors);
-			UNPROTECT(1); /* factors */
-
-			if (ivalid == 3) {
-				/* corMatrix */
-				SEXP sd = PROTECT(GET_SLOT(from, Matrix_sdSym));
-				if (LENGTH(sd) > 0)
-					SET_SLOT(to, Matrix_sdSym, sd);
-				UNPROTECT(1); /* sd */
-			}
-		}
-	}
-
-	char ul = *CHAR(STRING_ELT(uplo, 0));
-	if (ul != 'U')
-		SET_SLOT(to, Matrix_uploSym, uplo);
-	UNPROTECT(1); /* uplo */
-
-	SEXPTYPE tx;
-	R_xlen_t nx = PM_LENGTH(n);
-	SEXP x_from = PROTECT(GET_SLOT(from, Matrix_xSym)),
-	x_to = PROTECT(allocVector(tx = TYPEOF(x_from), nx));
-
-#define PACK(_PREFIX_, _PTR_) \
-	_PREFIX_ ## dense_pack(_PTR_(x_to), _PTR_(x_from), n, ul, 'N')
-
-	switch (tx) {
-	case LGLSXP: /* [ln]..Matrix */
-		PACK(i, LOGICAL);
-		break;
-	case INTSXP: /* i..Matrix */
-		PACK(i, INTEGER);
-		break;
-	case REALSXP: /* d..Matrix */
-		PACK(d, REAL);
-		break;
-	case CPLXSXP: /* z..Matrix */
-		PACK(z, COMPLEX);
-		break;
-	default:
-		ERROR_INVALID_TYPE("'x' slot", tx, "unpackedMatrix_pack");
-		break;
-	}
-
-#undef PACK
-
-	SET_SLOT(to, Matrix_xSym, x_to);
-
-	UNPROTECT(3); /* x_to, x_from, to */
-	return to;
-}
-
 /* forceSymmetric(x, uplo), returning .syMatrix */
 SEXP unpackedMatrix_force_symmetric(SEXP from, SEXP uplo_to)
 {
@@ -559,7 +456,7 @@ SEXP unpackedMatrix_force_symmetric(SEXP from, SEXP uplo_to)
 	/* 6 */ "dsyMatrix", "lsyMatrix", "nsyMatrix", ""};
 	int ivalid = R_check_class_etc(from, valid);
 	if (ivalid < 0)
-		ERROR_INVALID_CLASS(from, "unpackedMatrix_force_symmetric");
+		ERROR_INVALID_CLASS(from, __func__);
 	const char *clf = valid[ivalid];
 
 	char ulf = 'U', ult = 'U';
@@ -574,7 +471,7 @@ SEXP unpackedMatrix_force_symmetric(SEXP from, SEXP uplo_to)
 	    (TYPEOF(uplo_to) != STRSXP || LENGTH(uplo_to) < 1 ||
 	    (uplo_to = STRING_ELT(uplo_to, 0)) == NA_STRING ||
 	    ((ult = *CHAR(uplo_to)) != 'U' && ult != 'L')))
-		error(_("invalid 'uplo' to 'unpackedMatrix_force_symmetric()'"));
+		error(_("invalid '%s' to %s()"), "uplo", __func__);
 
 	if (clf[1] == 's') {
 		/* .syMatrix */
@@ -650,7 +547,7 @@ SEXP unpackedMatrix_force_symmetric(SEXP from, SEXP uplo_to)
 			COPY_DIAGONAL(z, Rcomplex, COMPLEX);
 			break;
 		default:
-			ERROR_INVALID_TYPE("'x' slot", tx, "unpackedMatrix_force_symmetric");
+			ERROR_INVALID_TYPE(x_from, __func__);
 			break;
 		}
 
@@ -664,7 +561,7 @@ SEXP unpackedMatrix_force_symmetric(SEXP from, SEXP uplo_to)
 	return to;
 }
 
-#define UPM_IS_TR(_RES_, _X_, _N_, _UPLO_, _WHAT_, _METHOD_) \
+#define UPM_IS_TR(_RES_, _X_, _N_, _UPLO_) \
 do { \
 	switch (TYPEOF(_X_)) { \
 	case LGLSXP: \
@@ -680,13 +577,13 @@ do { \
 		_RES_ = zdense_unpacked_is_triangular(COMPLEX(_X_), _N_, _UPLO_); \
 		break; \
 	default: \
-		ERROR_INVALID_TYPE(_WHAT_, TYPEOF(_X_), _METHOD_); \
+		ERROR_INVALID_TYPE(_X_, __func__); \
 		_RES_ = FALSE; \
 		break; \
 	} \
 } while (0)
 
-#define UPM_IS_SY(_RES_, _X_, _N_, _LDENSE_, _WHAT_, _METHOD_) \
+#define UPM_IS_SY(_RES_, _X_, _N_, _LDENSE_) \
 do { \
 	switch (TYPEOF(_X_)) { \
 	case LGLSXP: \
@@ -704,13 +601,13 @@ do { \
 		_RES_ = zdense_unpacked_is_symmetric(COMPLEX(_X_), _N_); \
 		break; \
 	default: \
-		ERROR_INVALID_TYPE(_WHAT_, TYPEOF(_X_), _METHOD_); \
+		ERROR_INVALID_TYPE(_X_, __func__); \
 		_RES_ = FALSE; \
 		break; \
 	} \
 } while (0)
 
-#define UPM_IS_DI(_RES_, _X_, _N_, _WHAT_, _METHOD_) \
+#define UPM_IS_DI(_RES_, _X_, _N_) \
 do { \
 	switch (TYPEOF(_X_)) { \
 	case LGLSXP: \
@@ -726,29 +623,28 @@ do { \
 		_RES_ = zdense_unpacked_is_diagonal(COMPLEX(_X_), _N_); \
 		break; \
 	default: \
-		ERROR_INVALID_TYPE(_WHAT_, TYPEOF(_X_), _METHOD_); \
+		ERROR_INVALID_TYPE(_X_, __func__); \
 		_RES_ = FALSE; \
 		break; \
 	} \
 } while (0)
 
-#define RETURN_GE_IS_TR(_X_, _N_, _UPPER_, _NPROT_, _WHAT_, _METHOD_) \
+#define RETURN_GE_IS_TR(_X_, _N_, _UPPER_, _NPROT_) \
 do { \
 	Rboolean res = FALSE; \
 	if (_UPPER_ == NA_LOGICAL) { \
-		UPM_IS_TR(res, _X_, _N_, 'U', _WHAT_, _METHOD_); \
+		UPM_IS_TR(res, _X_, _N_, 'U'); \
 		if (res) { \
 			UNPROTECT(_NPROT_); \
 			RETURN_TRUE_OF_KIND("U"); \
 		} \
-		UPM_IS_TR(res, _X_, _N_, 'L', _WHAT_, _METHOD_); \
+		UPM_IS_TR(res, _X_, _N_, 'L'); \
 		if (res) { \
 			UNPROTECT(_NPROT_); \
 			RETURN_TRUE_OF_KIND("L"); \
 		} \
 	} else { \
-		UPM_IS_TR(res, _X_, _N_, (_UPPER_ != 0) ? 'U' : 'L', \
-		          _WHAT_, _METHOD_); \
+		UPM_IS_TR(res, _X_, _N_, (_UPPER_ != 0) ? 'U' : 'L'); \
 	} \
 	UNPROTECT(_NPROT_); \
 	return ScalarLogical(res); \
@@ -763,7 +659,7 @@ SEXP unpackedMatrix_is_triangular(SEXP obj, SEXP upper)
 	/* 6 */ "dsyMatrix", "lsyMatrix", "nsyMatrix", ""};
 	int ivalid = R_check_class_etc(obj, valid);
 	if (ivalid < 0)
-		ERROR_INVALID_CLASS(obj, "unpackedMatrix_is_triangular");
+		ERROR_INVALID_CLASS(obj, __func__);
 
 	int need_upper = asLogical(upper);
 
@@ -775,8 +671,7 @@ SEXP unpackedMatrix_is_triangular(SEXP obj, SEXP upper)
 		if (!s)
 			return ScalarLogical(0);
 		SEXP x = PROTECT(GET_SLOT(obj, Matrix_xSym));
-		RETURN_GE_IS_TR(x, n, need_upper, /* unprotect this many: */ 1,
-		                "'x' slot", "unpackedMatrix_is_triangular");
+		RETURN_GE_IS_TR(x, n, need_upper, /* unprotect this many: */ 1);
 	} else {
 		/* .(tr|sy)Matrix */
 		SEXP uplo = PROTECT(GET_SLOT(obj, Matrix_uploSym));
@@ -788,8 +683,7 @@ SEXP unpackedMatrix_is_triangular(SEXP obj, SEXP upper)
 		SEXP x = PROTECT(GET_SLOT(obj, Matrix_xSym)), \
 			dim = PROTECT(GET_SLOT(obj, Matrix_DimSym)); \
 		int n = INTEGER(dim)[0]; \
-		UPM_IS_TR(res, x, n, (ul == 'U') ? 'L' : 'U', \
-		          "'x' slot", "unpackedMatrix_is_triangular"); \
+		UPM_IS_TR(res, x, n, (ul == 'U') ? 'L' : 'U'); \
 		UNPROTECT(2); /* dim, x */ \
 		if (res)
 
@@ -829,8 +723,7 @@ SEXP matrix_is_triangular(SEXP obj, SEXP upper)
 	if (!s)
 		return ScalarLogical(0);
 	int need_upper = asLogical(upper);
-	RETURN_GE_IS_TR(obj, n, need_upper, /* unprotect this many: */ 0,
-	                "matrix", "matrix_is_triangular");
+	RETURN_GE_IS_TR(obj, n, need_upper, /* unprotect this many: */ 0);
 }
 
 #undef RETURN_GE_IS_TR
@@ -845,7 +738,7 @@ SEXP unpackedMatrix_is_symmetric(SEXP obj, SEXP checkDN)
 	/* 6 */ "dsyMatrix", "lsyMatrix", "nsyMatrix", ""};
 	int ivalid = R_check_class_etc(obj, valid);
 	if (ivalid < 0) {
-		ERROR_INVALID_CLASS(obj, "unpackedMatrix_is_symmetric");
+		ERROR_INVALID_CLASS(obj, __func__);
 		return R_NilValue;
 	} else if (ivalid < 6) {
 		/* .(ge|tr)Matrix */
@@ -863,15 +756,13 @@ SEXP unpackedMatrix_is_symmetric(SEXP obj, SEXP checkDN)
 		SEXP x = PROTECT(GET_SLOT(obj, Matrix_xSym));
 		if (ivalid < 3) {
 			/* .geMatrix: need to do a complete symmetry check */
-			UPM_IS_SY(res, x, n, ivalid == 1,
-			          "'x' slot", "unpackedMatrix_is_symmetric");
+			UPM_IS_SY(res, x, n, ivalid == 1);
 		} else {
 			/* .trMatrix: symmetric iff diagonal (upper _and_ lower tri.) */
 			SEXP uplo = PROTECT(GET_SLOT(obj, Matrix_uploSym));
 			char ul = (*CHAR(STRING_ELT(uplo, 0)) == 'U') ? 'L' : 'U';
 			UNPROTECT(1); /* uplo */
-			UPM_IS_TR(res, x, n, ul,
-			          "'x' slot", "unpackedMatrix_is_symmetric");
+			UPM_IS_TR(res, x, n, ul);
 		}
 		UNPROTECT(1); /* x */
 		return ScalarLogical(res);
@@ -895,7 +786,7 @@ SEXP matrix_is_symmetric(SEXP obj, SEXP checkDN)
 	if (!s)
 		return ScalarLogical(0);
 	Rboolean res = FALSE;
-	UPM_IS_SY(res, obj, n, 1, "matrix", "matrix_is_symmetric");
+	UPM_IS_SY(res, obj, n, 1);
 	return ScalarLogical(res);
 }
 
@@ -908,7 +799,7 @@ SEXP unpackedMatrix_is_diagonal(SEXP obj)
 	/* 6 */ "dsyMatrix", "lsyMatrix", "nsyMatrix", ""};
 	int ivalid = R_check_class_etc(obj, valid);
 	if (ivalid < 0)
-		ERROR_INVALID_CLASS(obj, "unpackedMatrix_is_diagonal");
+		ERROR_INVALID_CLASS(obj, __func__);
 	SEXP dim = PROTECT(GET_SLOT(obj, Matrix_DimSym));
 	int *pdim = INTEGER(dim), n = pdim[0], s = pdim[1] == n;
 	UNPROTECT(1); /* dim */
@@ -918,13 +809,13 @@ SEXP unpackedMatrix_is_diagonal(SEXP obj)
 	SEXP x = PROTECT(GET_SLOT(obj, Matrix_xSym));
 	if (ivalid < 3) {
 		/* .geMatrix: need to do a complete diagonality check */
-		UPM_IS_DI(res, x, n, "'x' slot", "unpackedMatrix_is_diagonal");
+		UPM_IS_DI(res, x, n);
 	} else {
 		/* .(tr|sy)Matrix: diagonal iff stored triangle is zero off diagonal */
 		SEXP uplo = PROTECT(GET_SLOT(obj, Matrix_uploSym));
 		char ul = (*CHAR(STRING_ELT(uplo, 0)) == 'U') ? 'L' : 'U';
 		UNPROTECT(1); /* uplo */
-		UPM_IS_TR(res, x, n, ul, "'x' slot", "unpackedMatrix_is_diagonal");
+		UPM_IS_TR(res, x, n, ul);
 	}
 	UNPROTECT(1); /* x */
 	return ScalarLogical(res);
@@ -939,7 +830,7 @@ SEXP matrix_is_diagonal(SEXP obj)
 	if (!s)
 		return ScalarLogical(0);
 	Rboolean res = FALSE;
-	UPM_IS_DI(res, obj, n, "matrix", "matrix_is_diagonal");
+	UPM_IS_DI(res, obj, n);
 	return ScalarLogical(res);
 }
 
@@ -959,7 +850,7 @@ SEXP unpackedMatrix_transpose(SEXP from)
 	/* 8 */ "dsyMatrix", "lsyMatrix", "nsyMatrix", ""};
 	int ivalid = R_check_class_etc(from, valid);
 	if (ivalid < 0)
-		ERROR_INVALID_CLASS(from, "unpackedMatrix_transpose");
+		ERROR_INVALID_CLASS(from, __func__);
 	const char *cl = valid[ivalid];
 
 	SEXP to = PROTECT(NEW_OBJECT_OF_CLASS(cl));
@@ -1045,7 +936,7 @@ SEXP unpackedMatrix_transpose(SEXP from)
 		UPM_T(Rcomplex, COMPLEX);
 		break;
 	default:
-		ERROR_INVALID_TYPE("'x' slot", tx, "unpackedMatrix_transpose");
+		ERROR_INVALID_TYPE(x_from, __func__);
 		break;
 	}
 
@@ -1062,7 +953,7 @@ SEXP unpackedMatrix_diag_get(SEXP obj, SEXP nms)
 {
 	int do_nms = asLogical(nms);
 	if (do_nms == NA_LOGICAL)
-		error(_("'names' must be TRUE or FALSE"));
+		error(_("'%s' must be %s or %s"), "names", "TRUE", "FALSE");
 
 	SEXP dim = PROTECT(GET_SLOT(obj, Matrix_DimSym));
 	int *pdim = INTEGER(dim),
@@ -1115,7 +1006,7 @@ SEXP unpackedMatrix_diag_get(SEXP obj, SEXP nms)
 		UPM_D_G(Rcomplex, COMPLEX, Matrix_zone);
 		break;
 	default:
-		ERROR_INVALID_TYPE("'x' slot", tx, "unpackedMatrix_diag_get");
+		ERROR_INVALID_TYPE(x, __func__);
 		break;
 	}
 
@@ -1153,7 +1044,7 @@ SEXP unpackedMatrix_diag_set(SEXP obj, SEXP val)
 	/* 6 */ "dsyMatrix", "lsyMatrix", "nsyMatrix", ""};
 	int ivalid = R_check_class_etc(obj, valid);
 	if (ivalid < 0)
-		ERROR_INVALID_CLASS(obj, "unpackedMatrix_diag_set");
+		ERROR_INVALID_CLASS(obj, __func__);
 
 	SEXP dim = PROTECT(GET_SLOT(obj, Matrix_DimSym));
 	int *pdim = INTEGER(dim),
@@ -1232,7 +1123,7 @@ SEXP unpackedMatrix_diag_set(SEXP obj, SEXP val)
 		UPM_D_S(Rcomplex, COMPLEX);
 		break;
 	default:
-		ERROR_INVALID_TYPE("'x' slot", tx, "unpackedMatrix_diag_set");
+		ERROR_INVALID_TYPE(x, __func__);
 		break;
 	}
 
@@ -1253,7 +1144,7 @@ SEXP unpackedMatrix_symmpart(SEXP from)
 	/* 6 */ "dsyMatrix", "lsyMatrix", "nsyMatrix", ""};
 	int ivalid = R_check_class_etc(from, valid);
 	if (ivalid < 0)
-		ERROR_INVALID_CLASS(from, "unpackedMatrix_symmpart");
+		ERROR_INVALID_CLASS(from, __func__);
 
 	const char *clf = valid[ivalid];
 	if (clf[0] == 'd' && clf[1] == 's')
@@ -1429,7 +1320,7 @@ SEXP matrix_symmpart(SEXP from)
 		break;
 #endif
 	default:
-		ERROR_INVALID_TYPE("matrix", TYPEOF(x), "matrix_symmpart");
+		ERROR_INVALID_TYPE(x, __func__);
 		break;
 	}
 
@@ -1459,7 +1350,7 @@ SEXP unpackedMatrix_skewpart(SEXP from)
 	/* 6 */ "dsyMatrix", "lsyMatrix", "nsyMatrix", ""};
 	int ivalid = R_check_class_etc(from, valid);
 	if (ivalid < 0)
-		ERROR_INVALID_CLASS(from, "unpackedMatrix_skewpart");
+		ERROR_INVALID_CLASS(from, __func__);
 	const char *clf = valid[ivalid];
 
 	char clt[] = "...Matrix";
@@ -1657,7 +1548,7 @@ SEXP matrix_skewpart(SEXP from)
 		break;
 #endif
 	default:
-		ERROR_INVALID_TYPE("matrix", TYPEOF(x), "matrix_skewpart");
+		ERROR_INVALID_TYPE(x, __func__);
 		break;
 	}
 
