@@ -1,32 +1,37 @@
 ####--------- Test interfaces to other non-standard Packages ---------------
 
+## for R_DEFAULT_PACKAGES=NULL :
+library(grDevices)
+library(stats)
+library(utils)
+
 library(Matrix)
-
 source(system.file("test-tools.R", package = "Matrix"))# identical3() etc
-
 MatrixRversion <- pkgRversion("Matrix")
 
+
 ###-- 1)  'graph' (from Bioconductor) ---------------------------
 ###-- ==  =======                     ---------------------------
-## do not run the test "usually" for now [Solaris problem after detach() ..]:
-if((Sys.getenv("USER")=="maechler" || nzchar(Sys.getenv("R_MATRIX_CHECK_EXTRA"))) &&
-   isTRUE(try(require(graph)))) { # may be there and fail (with R-devel)
 
-    if(packageDescription("graph")$Version <= "1.10.2") {
-        ## graph 1.10.x for x <= 2 had too many problems  as(<graph>, "matrix")
-        cat("Version of 'graph' is too old --- no tests done here!\n")
+if(requireNamespace("graph")) {
+
+    if(packageVersion("graph") <= "1.10.2") {
+
+    ## graph 1.10.x for x <= 2 had too many problems  as(<graph>, "matrix")
+    cat("Version of 'graph' is too old --- no tests done here!\n")
 
     } else if(pkgRversion("graph") != MatrixRversion) {
 
-        cat(sprintf("The R version (%s) of 'graph' installation differs from the Matrix one (%s)\n",
-                    pkgRversion("graph"), MatrixRversion))
+    cat(sprintf("The R version (%s) of 'graph' installation differs from the Matrix one (%s)\n",
+                pkgRversion("graph"), MatrixRversion))
 
     } else { ## do things
 
-    if(!dev.interactive(orNone=TRUE)) pdf("other-pkg-graph.pdf")
+    if(doPdf <- !dev.interactive(orNone = TRUE))
+        pdf("other-pkgs-graph.pdf")
 
     ## 0) Simplest non-trivial graph: has no weights:
-    g0 <- graphNEL(paste(1:2), edgeL=list("1"="2"), "directed")
+    g0 <- graph::graphNEL(paste(1:2), edgeL=list("1"="2"), "directed")
     m0 <- as(g0, "Matrix")
     stopifnot(is(m0,"ngCMatrix"), dim(m0) == c(2,2), which(m0) == 3)
     g. <- as(m0, "graph") ## failed in Matrix <= 1.1-0
@@ -42,7 +47,7 @@ if((Sys.getenv("USER")=="maechler" || nzchar(Sys.getenv("R_MATRIX_CHECK_EXTRA"))
     for(i in 1:4)
         edL[[i]] <- list(edges = 5-i)
     gR <- new("graphNEL", nodes=V, edgeL=edL)
-    str(edges(gR))
+    str(graph::edges(gR))
     sm.g <- as(gR, "sparseMatrix")
     str(sm.g) ## dgC: TODO: want 'ds.' (symmetric)
     validObject(sm.g)
@@ -53,21 +58,21 @@ if((Sys.getenv("USER")=="maechler" || nzchar(Sys.getenv("R_MATRIX_CHECK_EXTRA"))
     for(i in 1:4)
         edL[[i]] <- list(edges = 5-i, weights=runif(1))
     gRw <- new("graphNEL", nodes=V, edgeL=edL)
-    str(edgeWeights(gRw))
+    str(graph::edgeWeights(gRw))
     sm.gw <- as(gRw, "sparseMatrix")
     str(sm.gw) ## *numeric* dgCMatrix
     validObject(sm.gw)
     show( sm.gw )## U[0,1] numbers in anti-diagonal
 
     ## 2) directed
-    gU <- gR; edgemode(gU) <- "directed"
+    gU <- gR; graph::edgemode(gU) <- "directed"
     sgU <- as(gU, "sparseMatrix")
     str(sgU) ## 'dgC'
     validObject(sgU)
     show( sgU )
 
     ## Reverse :  sparseMatrix -> graph
-    sm.g[1,2] <- 1
+    sm.g[1,2] <- TRUE
     gmg  <-  as(sm.g, "graph")
     validObject(gmg2 <-  as(sm.g, "graphNEL"))
     gmgw <-  as(sm.gw, "graph")
@@ -78,37 +83,38 @@ if((Sys.getenv("USER")=="maechler" || nzchar(Sys.getenv("R_MATRIX_CHECK_EXTRA"))
               identical(gmgw, gmgw2),
               identical(gmgU, gmgU2))
 
-    data(CAex)
+    data(CAex, package = "Matrix")
     cc <- crossprod(CAex)
     ## work around bug in 'graph': diagonal must be empty:
     diag(cc) <- 0; cc <- drop0(cc)
     image(cc)
     gg <- as(cc, "graph")
 
-    .r <- require# cheat checks - do *not* want it in DESCRIPTION:
-    if(.r("Rgraphviz")) {
-	plot(gg, "circo")
-	detach("package:Rgraphviz", unload = TRUE)
-    }
-    stopifnot(all.equal(edgeMatrix(gg),
+    ## Don't run on CRAN and don't trigger 'R CMD check' :
+    if(doExtras && match.fun("requireNamespace")("Rgraphviz"))
+        get("plot", asNamespace("Rgraphviz"), mode = "function")(gg, "circo")
+
+    stopifnot(all.equal(graph::edgeMatrix(gg),
                         rbind(from = c(rep(1:24, each=2), 25:48),
                               to   = c(rbind(25:48,49:72), 49:72))))
 
-    detach("package:graph", unload = TRUE)
-    dev.off()
+    if(doPdf)
+        dev.off()
+
     } # {else}
 
-} ## end{graph}
+} # end{graph}
+
 
 ###-- 2)  'SparseM' ---------------------------------------------
-###-- ==  ========  ---------------------------------------------
+###-- ==  ========= ---------------------------------------------
 
-if(isTRUE(try(require(SparseM)))) { # may be there and fail
+if(requireNamespace("SparseM")) {
 
     if(pkgRversion("SparseM") != MatrixRversion) {
 
-        cat(sprintf("The R version (%s) of 'SparseM' installation differs from the Matrix one (%s)\n",
-                    pkgRversion("SparseM"), MatrixRversion))
+    cat(sprintf("The R version (%s) of 'SparseM' installation differs from the Matrix one (%s)\n",
+                pkgRversion("SparseM"), MatrixRversion))
 
     } else { ## do things
 
@@ -118,9 +124,9 @@ if(isTRUE(try(require(SparseM)))) { # may be there and fail
 	A <- matrix(a,5,4)
 	print(M <- Matrix(A))
 	stopifnot(
-            validObject(A.csr <- as.matrix.csr(A)),
-            validObject(At.csr <- as.matrix.csr(t(A))),
-            validObject(A.csc <- as.matrix.csc(A)),
+            validObject(A.csr <- SparseM::as.matrix.csr(A)),
+            validObject(At.csr <- SparseM::as.matrix.csr(t(A))),
+            validObject(A.csc <- SparseM::as.matrix.csc(A)),
             identical(At.csr, t(A.csr)),
             identical(A, as.matrix(A.csr)),
             identical(A.csr, as(M, "matrix.csr")),
@@ -130,7 +136,7 @@ if(isTRUE(try(require(SparseM)))) { # may be there and fail
             )
 
 	## More tests, notably for triplets
-	A.coo <- as.matrix.coo(A)
+	A.coo <- SparseM::as.matrix.coo(A)
 	str(T  <- as(M, "TsparseMatrix")) # has 'j' sorted
 	str(T. <- as(A.coo, "TsparseMatrix")) # has 'i' sorted
 
@@ -138,13 +144,10 @@ if(isTRUE(try(require(SparseM)))) { # may be there and fail
 	M3 <- as(A.csr,               "Matrix") # dgC
 	M4 <- as(A.csc,               "Matrix") # dgC
 	M5 <- as(as(M, "matrix.coo"), "Matrix") # dgT
-	uniqT <- Matrix:::uniqTsparse
+	uniqT <- uniqTsparse
 	stopifnot(identical4(uniqT(T), uniqT(T.), uniqT(T3), uniqT(M5)),
 		  identical3(M, M3, M4))
 
-	if(FALSE) # detaching the package gives error ".GenericTable" not found
-	    detach("package:SparseM")
+    } # {else}
 
-    }
-
-}## end{SparseM}
+} # end{SparseM}
